const path = require("path")
const webpack = require("webpack")
const _ = require('lodash')
const util = require('../util')
const chalk = require('chalk')
const MiniCssExtractPlugin = require('mini-css-extract-plugin')


const dirname = process.cwd() // 当前目录
const userConfig = readUserConfig(dirname)
const { webpackConfig, templates } = generateWebpackConfig(userConfig)
function readUserConfig (dirname) {
  if (process.argv.length > 2 && process.argv[2]) {
    console.log(`构建页面：${process.argv[2]}`)
    const configFilePath = path.resolve(dirname, 'cli-config', `index.js`)
    if (util.isFile(configFilePath)) {
      return require(configFilePath)[process.argv[2]] || {}
    }
  }
  return util.isFile(path.resolve(dirname, 'sy.config.js')) ? require(path.resolve(dirname, 'sy.config'))
    : require(path.resolve(dirname, 'webpack.config'))
}

function generateWebpackConfig ({ context = dirname, runtimeOnly = true, files, exclude, index = path.join(__dirname, 'template.html') }) {
  if (!files) {
    throw new Error('编译失败,没有可编译的模块，检查files入口文件配置')
  }

  // 格式化files配置
  files = handleFiles(files, index)
  // console.log(files)
  const entry = {}
  const templates = []
  _.each(files, (templatePath, filePath) => {
    // console.log(filePath)
    if (!_.includes(filePath.split('/'), 'src')) {
      console.error(chalk.bold.red('\n编译失败,' + filePath + '中没有src目录\n'))
      process.exit(-1)
    }
    solveEntry(entry, templates, filePath, templatePath)
  })

  // 排除的目录
  exclude = _.isObject(exclude) ? exclude : exclude ? /node_modules/ : undefined
  // console.log(exclude, path.resolve(context), entry, context, templates)
  const webpackConfig = {
    // 根目录,entry的入口文件
    context,
    // string | object | array
    entry,
    output: {
      // 静态文件访问路径
      // publicPath: './',
      // 输出文件的地址
      path: context,
      // 输出的文件名 hash统一生成,chunkhash变化生成
      filename: '[name].js',
      // 输出块的名字
      chunkFilename: '[name].[chunkhash:8].js',
      assetModuleFilename: templates[0].basePath+'/asset/[name]-[contenthash][ext][query]'
    },
    resolve: {
      extensions: ['.js', '.json', '.vue', '.jsx', '.ts'],
      alias: {
        'vue$': `vue/dist/vue.${runtimeOnly === false ? '' : 'runtime.'}esm.js`,
        'components': path.join(context, 'components'),
        'pages': path.join(context, 'pages'),
        '@': path.resolve(context),
        'crypto': false
      },
      fallback: {
        "path": require.resolve("path-browserify"),
        // "crypto": require.resolve("crypto-browserify")
      }
    },
    module: {
      rules: [
        {
          test: /\.vue$/,
          use: [
            {
              loader: 'vue-loader'
            }
          ]
        },
        {
          test: /\.(js|jsx)$/,
          exclude: exclude,
          use: [ {
            loader: 'babel-loader',
            options: {
              cacheDirectory: true
            }
          }]
        },
        {
          test: /\.ts?$/,
          loader: 'ts-loader',
          exclude: exclude,
          options: {
            appendTsSuffixTo: [/\.vue$/],
          }
        },
        {
          test: /\.(jpe?g|png|svg|gif|webp)$/i,
          type: 'asset/resource',
          generator: {
            filename: templates[0].basePath+'/img/[name]-[contenthash][ext][query]'
          },
        },
        {
          test: /\.(woff|svg|eot|ttf)\??.*$/,
          type: 'asset/resource'
        },
        {
          exclude: [
            /\.vue$/,
            /\.(js|jsx)$/,
            /\.css$/,
            /\.styl$/,
            /\.sass$/,
            /\.scss$/,
            /\.less$/,
            /\.html$/,
            /\.json$/,
            /\.(woff|svg|eot|ttf)\??.*$/,
            /\.(jpe?g|png|svg|gif|webp)$/i,
            /\.hbs$/
          ],
          type: 'asset/resource'
        }
      ]
    },
    plugins: [
    ]
  }
  return {
    webpackConfig,
    templates
  }
}

// 处理文件入口路径问题, 处理不是很严谨
function solveEntry (entry, templates, filePath, templatePath) {
  let entryPath = filePath
  if (_.includes(filePath, '.')) {
    entryPath = entryPath.substring(0, entryPath.lastIndexOf('.'))
  }

  // const index = entryPath.indexOf('src')
  entryPath = entryPath.replace('src', 'dist')
  // const filename = index > 0 ? entryPath.substring(0, index - 1) : 'index'

  entry[entryPath] = './' + filePath
  templates.push({
    filePath,
    filename: entryPath + '.html',
    template: templatePath,
    chunk: entryPath,
    basePath: entryPath.substring(0, entryPath.indexOf('dist') + 4)
  })
}

// 将文件处理成统一格式
function handleFiles (files, index) {
  const res = {}
  _.each(files, (item, key) => {
    // 对象形式
    if (_.isString(key)) {
      if (item === 1) {
        res[key] = index
      } else if (_.isString(item)) {
        res[key] = item
      }
    } else {
      if (_.isString(item)) {
        res[item] = index
      } else if (_.isArray(item)) {
        res[item[0]] = item[1]
      }
    }
  })

  return res
}

module.exports = {
  userConfig,
  webpackConfig,
  templates
}
